/***************************************************
  mcmr.h : contains declarations for the time-dependent
           mcmr simulator 
****************************************************/
#ifndef __MCMR_H
#define __MCMR_H

/* interface functions */
extern int Print_C();
extern int Print_R();

extern void Init_Q_Module();
extern void Evaluate();
extern void Done_Q_Module();

extern void Set_Config_i(int nType, int nValue);
extern void Set_Config_d(int nType, double dValue);
extern int Add_Resource(unsigned int nResourceID, double dMaxCapacity);
extern int Add_Class(unsigned int nClassID, double dArrRate);
extern int Add_Class_At_Resource(unsigned int nClassID, 
                                 unsigned int nResourceID,
                            double dRequire, double dSerRate);
extern void Remove_Resource(unsigned int nResourceID);
extern void Remove_Class(unsigned int nClassID);
extern int Add_Callback(int (*pCallBack)( double, int, void *),
                      double time, int nParam, void *pParam,
                      unsigned int *pCallBackID);
extern void Remove_Callback(unsigned int nCallBackID);
extern void Set_Arrival_Rate(unsigned int nClassID, double dNewRate, double *dOldRate);
extern void Set_Service_Rate(unsigned int nClassID, 
                             unsigned int nResourceID, double dNewRate, 
                             double *dOldRate);
extern void Register_Dump_Function( int (*pCallBack)(double, int, 
                                double**, double**, double, void *), void * );
extern int Fetch_Avg_Metrics(int nMeasureID, unsigned int nClassID,
                        unsigned int nResourceID, double *pValue);
extern int Fetch_Crt_Metrics(int nMeasureID, unsigned int nClassID,
                        unsigned int nResourceID, double *pValue);
extern void Print_Mcmr_Error(int nErrcode);
extern void Recompute_Resource_Utilization();
extern void Remove_Class_From_Resource( unsigned int nClassID, unsigned int nResourceID);

/* error constants */
#define ERR_SUCC                        0
#define ERR_ADD_SUCCESS                 0
#define ERR_FETCH_SUCCESS               0

#define ERR_ADD_ALLOC                  -1
#define ERR_CALLBACK_NOTINITIALIZED    -2
#define ERR_CALLBACK_BADTIME           -3
#define ERR_CALLBACK_ALLOC             -4
#define ERR_FETCH_NOMETRIC             -5

#define ERR_INVALID_RESOURCEID         - 6   
#define ERR_INVALID_CLASSID            - 7
#define ERR_INVALID_CONFIG_PARAM       - 8   
#define ERR_DUP_RESOURCEID             - 9
#define ERR_DUP_CLASSID                - 10
#define ERR_INVALID_METRICID           - 11
#define ERR_INVALID_CALLBACKID         - 12

/* metric constants */
#define METRIC_BLOCKING_PROB_CLASS      1
#define METRIC_BLOCKING_PROB            2
#define METRIC_NUMBER_CLASS             3
#define METRIC_UTILIZATION              4

/* only for simulation */
#define METRIC_ACCEPTED                 100
#define METRIC_ACCEPTED_CLASS           101
#define METRIC_DEPART                   102
#define METRIC_DEPART_CLASS             103 
#define METRIC_BLOCKED                  104
#define METRIC_BLOCKED_CLASS            105
#define METRIC_NUMBER                   106
#define METRIC_UTILIZATION_CLASS        107

/* config constants */
#define CONFIG_ARRIVAL_DISTRIBUTION     1
#define CONFIG_SERVICE_DISTRIBUTION     2
#define CONFIG_NUMBER_OF_SIMULATIONS    3
#define CONFIG_SIMULATION_TIME          4
#define CONFIG_DUMP_PERIOD              5
#define CONFIG_EVALUATION_PERIOD        6   
#define CONFIG_EPSILON_STOP             7   

#define CONFIG_POISSON                  100 

#endif
