#include <stdio.h>
#include <assert.h>
#include <sys/time.h>
#include <stdlib.h>
#include <math.h>

#include "sys_mcmr.h"
#include "mcmr.h"

/* function declarations */
void InitEvent();
void AddEvent(int, int);
void DeleteEvent(event_type *);
event_type *FetchNextEvent();
event_type *GetNewIndex();
int AddCallBackEvent(int, double, int, void *);
void AdvanceEventlistHdr(event_type *);
void ResetEventlistHdr();

void AddEvent(int EventType, int ClassId)
{
    int j, k;
    resource_usage_type *ru;
    double time, rate;
	event_type *ev;


    switch (EventType) {
        case ARRIVAL :
            /* determine time */
            if (!Double_Equal(dLambda, 0.0)) {
                time = CTR.dCurrentTime + 
                        expntl(1 / dLambda);
            } else {
                break;
            }

            if (time >= CTR.dSimulationTime)
                break;

            /* determine the event list slot */
            ev = GetNewIndex(time);  
            assert(ev);

            eventtype(ev) = ARRIVAL;
            rate = drand48() * dLambda;
            j=0;
            while (Lambdas[j] < rate) {
                j++;
            }
            assert(j < C.nSetSize);
            eventclass(ev) = j;
            eventtimestamp(ev) = time;

            break;
            
        case DEPARTURE :
            /* NOTE : Assume every customer relinquishes its 
                      resources at once, thus, dSerRate has
                      same values for all the resources 
                      belong to this class
            */
#ifdef DEBUG
fprintf(stderr, "classID= %d, serrate=%f\n", ClassId, ru[0].dSerRate);
#endif
            /* determine time */
            ru = C.sResources[ClassId];
    		if ( ! Double_Equal(ru[0].dSerRate, 0.0) ) {
            /*if (ru[0].dSerRate != 0.0) {*/
                time = CTR.dCurrentTime + 
                        expntl(1 / ru[0].dSerRate);
            } else {
#ifdef DEBUG
fprintf(stderr, "!!!!!-----------------serrate = 0.0\n");
fflush(stderr);
#endif
                break;
            }
#ifdef DEBUG
fprintf(stderr, "CTR.dCurrentTime = %f, time = %f, ru[0].dSerRate = %f\n", 
CTR.dCurrentTime, time, ru[0].dSerRate);
#endif

            if (time >= CTR.dSimulationTime)
                break;

            /* determine the event list slot */
            ev = GetNewIndex(time);  
#ifdef DEBUG
if (i < 0)
fprintf(stderr, "time = %f, i=%d\n", time, i); 
#endif
            assert(ev);

            eventtype(ev) = DEPARTURE;
            eventclass(ev) = ClassId;    

            /* take the service rate of the first resource */ 
            eventtimestamp(ev) = time;

            break;

        default :
            assert(1);
            break;
    }
#ifdef DEBUG
fprintf(stderr, "At time = %f, i = %d\n", time, i);
#endif
}

void DeleteEvent(event_type *ev)
{
	event_type *pev, *nev;

	pev = ev->pPrev;
	nev = ev->pNext;
	
	if (nev) {
		nev->pPrev = pev;
	}
	if (pev) {
		pev->pNext = nev;	
	}
	
	free(ev);
}

/* should increase CTR.dCurrentTime */
event_type *FetchNextEvent()
{
	if (E.hp->pNext != NULL) { /* has something to do */
		CTR.dCurrentTime = E.hp->pNext->dTimeStamp;
	}

    return E.hp->pNext; 
}

/* can be different according to the event list algorithm */
event_type *GetNewIndex(double time)    
{
	event_type *ev, *pev, *tmp;

	/* search the right spot  */
	pev = ev = E.hp;
	while (ev) {
		if (eventtimestamp(ev) > time) { /* got it */
			break;
		}
		pev = ev;
		ev = ev->pNext;
	}

	tmp = (event_type *)malloc(sizeof(event_type));
	assert(tmp);

	pev->pNext = tmp; 
	tmp->pPrev = pev; 
	tmp->pNext = ev;

    return tmp;
}

void InitEvent()
{
	E.header.pNext = NULL;
	E.header.pPrev = NULL;
	E.hp = &(E.header);
}

int AddCallBackEvent(int CallBackId, double time, int nParam, void *pParam)
{
    int i;
	event_type *ev;
    cb_event_type *cep;

    if (time < 0.0 || time > CTR.dSimulationTime) {
        return -1;
    }

    ev = GetNewIndex(time);
    if (ev == NULL) {
        return -1;
    }

    if ( Double_Equal(CTR.dCurrentTime, 0.0) ) {
        eventtype(ev) = CALLBACKINIT;
    } else {
        eventtype(ev) = CALLBACK;
    }

    cep = (cb_event_type *)malloc(sizeof(cb_event_type));
    if (!cep) {
        return ERR_ADD_ALLOC;
    }
    cep->nId = CallBackId;
    cep->nParam = nParam;
    cep->pParam = pParam;
    (cb_event_type *)eventclass(ev) = cep;

    eventtimestamp(ev) = time;

    return 0;
}

/* For callbacks */
void AdvanceEventlistHdr(event_type *ev)
{
    E.hp = ev; 
}

void ResetEventlistHdr()
{
	E.hp = &(E.header);
}
