#include <memory.h>
#include <assert.h>
#include <stdio.h>
#include <sys/time.h>
#include <string.h>
#include "sys_mcmr.h"
#include "mcmr.h"

/*******************************************************
        Start/Stop using Q_module 
********************************************************/
void Init_Q_Module() {
    int i;

    /* configuration parameters */ 
    CTR.dCurrentTime = 0.0;
    CTR.nCurrentEventRecord = 0;
    CTR.nCurrentMeasureRecord = 0;

    /* init event list */
    InitEvent();
}

void Done_Q_Module() {

    FreeCountMeasure(0.0, CNT);
    FreeCountMeasure(0.0, CNT2);
    FreeC(&C);
    FreeR(&R);
    FreeC(&C2);
    FreeR(&R2);

}

/*******************************************************
        Set configuration parameters 
********************************************************/
void Set_Config_i(int nType, int nValue) {

    switch(nType) {
        case CONFIG_ARRIVAL_DISTRIBUTION:
            break;

        case CONFIG_SERVICE_DISTRIBUTION:
            break;

        case CONFIG_NUMBER_OF_SIMULATIONS :
            CTR.nSimulationRuns = nValue;
            break;

        default:
            return;
    }

    return;
}

void Set_Config_d(int nType, double dValue) {

    switch(nType) {

        case CONFIG_SIMULATION_TIME :
            CTR.dSimulationTime = dValue;
            break;

        case CONFIG_DUMP_PERIOD :
            CTR.dCollectStdMeasure = dValue;
            break;

        default:
            return;
    }

    return;
}

/*******************************************************
        Adding / Deleting resources and classes 
********************************************************/
/* add a single resource */
int Add_Resource(unsigned int nResourceID, double dMaxCapacity) 
{
    int i;

    /* 1. See if nResourceID is being used */
    for (i=0;i < R.nSetSize;i++) {
        if (R.id[i] == nResourceID) {
            return ERR_DUP_RESOURCEID;
        }
    }

    /* enlarge the R structure */
    R.nSetSize ++;

    R.id = (int *)realloc(R.id, R.nSetSize * sizeof(int));
    if (! R.id) {
        return ERR_ADD_ALLOC;
    }
    R.id[ R.nSetSize-1 ] = nResourceID;

    R.inuse = (int *)realloc(R.inuse, R.nSetSize * sizeof(int));
    if (! R.inuse) {
        return ERR_ADD_ALLOC;
    }
    R.inuse[ R.nSetSize-1 ] = INUSE;

    R.dCapacity = (double *)realloc(R.dCapacity, 
                                R.nSetSize * sizeof(double));
    if (! R.dCapacity) {
        return ERR_ADD_ALLOC;
    }
    R.dCapacity[ R.nSetSize-1 ] = dMaxCapacity;
    
    R.dAvailable = (double *)realloc(R.dAvailable, 
                                R.nSetSize * sizeof(double));
    if (! R.dAvailable) {
        return ERR_ADD_ALLOC;
    }
    R.dAvailable[ R.nSetSize-1 ] = dMaxCapacity;

    R.nClassId = (int **)realloc(R.nClassId, 
                                R.nSetSize * sizeof(int *));
    if (! R.nClassId) {
        return ERR_ADD_ALLOC;
    }
    R.nClassId[ R.nSetSize-1 ] = NULL;

    R.nNumClasses = (int *)realloc(R.nNumClasses, 
                                R.nSetSize * sizeof(int));
    if (! R.nNumClasses) {
        return ERR_ADD_ALLOC;
    }
    R.nNumClasses[ R.nSetSize-1 ] = 0;

    return ERR_SUCC;
}

/* Add a single class */
int Add_Class(unsigned int nClassID, double dArrRate)
{
    int i;

    /* 1. See if nClassID is being used */
    for (i=0;i < C.nSetSize;i++) {
        if (C.id[i] == nClassID) {
            return ERR_DUP_CLASSID;
        }
    }

    /* enlarge the C structure */
    C.nSetSize ++;

    C.id = (int *)realloc(C.id, C.nSetSize * sizeof(int));
    if (! C.id) {
        return ERR_ADD_ALLOC;
    }
    C.id[ C.nSetSize-1 ] = nClassID;

    C.inuse = (int *)realloc(C.inuse, C.nSetSize * sizeof(int));
    if (! C.inuse) {
        return ERR_ADD_ALLOC;
    }
    C.inuse[ C.nSetSize-1 ] = INUSE;

    C.dArrRate = (double *)realloc(C.dArrRate, C.nSetSize * sizeof(double));
    if (! C.dArrRate) {
        return ERR_ADD_ALLOC;
    }
    C.dArrRate[ C.nSetSize-1 ] = dArrRate;

    C.nNumResources = (int *)realloc(C.nNumResources, 
                                C.nSetSize * sizeof(int));
    if (! C.nNumResources) {
        return ERR_ADD_ALLOC;
    }
    C.nNumResources[ C.nSetSize-1 ] = 0;

    C.sResources = (resource_usage_type **)realloc(C.sResources, 
                                C.nSetSize * sizeof(resource_usage_type));
    if (! C.sResources) {
        return ERR_ADD_ALLOC;
    }
    C.sResources[ C.nSetSize-1 ] = NULL;

    BuildLambdaList();

    return ERR_SUCC;
}

/* associate the class with a resource */
int Add_Class_At_Resource(unsigned int nClassID, unsigned int nResourceID,
                            double dRequiredCapacity, double dSerRate) 
{
    int cidx, ridx, i;

    /* obtain resource and class */
    ridx = rid2idx(nResourceID);
    cidx = cid2idx(nClassID);

    if (cidx < 0) {
        return ERR_INVALID_CLASSID;
    }
    if (ridx < 0) {
        return ERR_INVALID_RESOURCEID;
    }

    for (i=0;i < C.nNumResources[cidx];i++) {
        if ( C.sResources[cidx][i].nIndex == nResourceID ) {
            break;
        }
    }
    /* case 1 : the user just wants to change the info */ 
    if ( i < C.nNumResources[cidx] ) {
        C.sResources[cidx][i].dRequire = dRequiredCapacity; 
        C.sResources[cidx][i].dSerRate = dSerRate; 
    } else {
    /* case 2 : new request */
        C.nNumResources[cidx] ++; 
        
        C.sResources[cidx] = (resource_usage_type *)realloc(
                                C.sResources[cidx],
                                C.nNumResources[cidx] *
                                sizeof(resource_usage_type));
        if (! C.sResources[cidx] ) {
            return ERR_ADD_ALLOC;
        }
        C.sResources[cidx][ C.nNumResources[cidx]-1 ].nIndex =
                        ridx;
        C.sResources[cidx][ C.nNumResources[cidx]-1 ].dRequire =
                        dRequiredCapacity;
        C.sResources[cidx][ C.nNumResources[cidx]-1 ].dSerRate =
                        dSerRate;

        if ( CTR.nCurrentRun == 0 && CTR.dCurrentTime != 0.0 ) {
            Check_CS();
            FreeCountMeasure(CTR.dCurrentTime, CNT); 
            FreeCountMeasure(CTR.dCurrentTime, CNT2); 
            AllocCountMeasure(CTR.dCurrentTime, &CNT); 
            AllocCountMeasure(CTR.dCurrentTime, &CNT2); 
        }
    } 

    return ERR_SUCC;
}

/* delete a single resource */
void Remove_Resource(unsigned int nResourceID)
{
    int ridx, cidx, i, j;

    /* get the idx */
    ridx = rid2idx(nResourceID);

    if (ridx < ERR_SUCC) {
        return;
    }

    /* remove it from class structure */
    for (i=0;i < R.nNumClasses[ridx];i++) {
        cidx = R.nClassId[ridx][i]; 

        /* trick : we are not shrinking the structure */
        for (j=0;j < C.nNumResources[cidx] - 1;j++) {
            if ( C.sResources[cidx][j].nIndex == ridx ) {
                /* shift the rest to the front */
                memmove( &(C.sResources[cidx][j]),
                         &(C.sResources[cidx][j+1]),
                         (C.nNumResources[cidx] - j - 1) *
                         sizeof(resource_usage_type) );
                break;
            }
        }
        C.nNumResources[cidx] --;
    }

    /* structure R */
    if (ridx < R.nSetSize - 1) { /* make sure it's not the last */
        memmove( &(R.id[ridx]), &(R.id[ridx+1]), 
                 (R.nSetSize - ridx - 1) * sizeof(int) );
        memmove( &(R.inuse[ridx]), &(R.inuse[ridx+1]), 
                 (R.nSetSize - ridx - 1) * sizeof(int) );
        memmove( &(R.dCapacity[ridx]), &(R.dCapacity[ridx+1]), 
                 (R.nSetSize - ridx - 1) * sizeof(double)            );
        memmove( &(R.dAvailable[ridx]), &(R.dAvailable[ridx+1]), 
                 (R.nSetSize - ridx - 1) * sizeof(double)            );
        memmove( R.nClassId[ridx], R.nClassId[ridx+1], 
                 (R.nSetSize - ridx - 1) * sizeof(int *)             );
        memmove( &(R.nNumClasses[ridx]), &(R.nNumClasses[ridx+1]), 
                 (R.nSetSize - ridx - 1) * sizeof(int)               );
    }
    R.nSetSize --;

    if ( CTR.nCurrentRun == 0 ) {
        Check_CS();
        FreeCountMeasure(CTR.dCurrentTime, CNT); 
        FreeCountMeasure(CTR.dCurrentTime, CNT2); 
        AllocCountMeasure(CTR.dCurrentTime, &CNT); 
        AllocCountMeasure(CTR.dCurrentTime, &CNT2); 
    }

#ifdef DEBUG
fprintf(stderr, "\nRemove_Resource() \n");
Print_R();
#endif
    
    return;
}

/* delete a single class */
void Remove_Class(unsigned int nClassID) 
{
    int ridx, cidx, i, j;

    /* get the idx */
    cidx = cid2idx(nClassID);

    if (cidx < ERR_SUCC) {
        return;
    }

    /* remove it from resource structure */
    for (i=0;i < C.nNumResources[cidx];i++) {
        ridx = C.sResources[cidx][i].nIndex; 

        /* trick : we are not shrinking the structure */
        for (j=0;j < R.nNumClasses[ridx] - 1;j++) {
            if ( R.nClassId[ridx][j] == cidx ) {
                /* shift the rest to the front */
                memmove( &(R.nClassId[ridx][j]),
                         &(R.nClassId[ridx][j+1]),
                         (R.nNumClasses[ridx] - j - 1) *
                         sizeof(int) );
                break;
            }
        }
        R.nNumClasses[ridx] --;
    }

    /* structure C */
    if (cidx < C.nSetSize - 1) { /* make sure it's not the last */
        memmove( &(C.id[cidx]), &(C.id[cidx+1]), 
                 (C.nSetSize - cidx - 1) * sizeof(int) );
        memmove( &(C.inuse[cidx]), &(C.inuse[cidx+1]), 
                 (C.nSetSize - cidx - 1) * sizeof(int) );
        memmove( &(C.dArrRate[cidx]), &(C.dArrRate[cidx+1]), 
                 (C.nSetSize - cidx - 1) * sizeof(double)            );
        memmove( &(C.nNumResources[cidx]), &(C.nNumResources[cidx+1]), 
                 (C.nSetSize - cidx - 1) * sizeof(int)               );
        memmove( C.sResources[cidx], C.sResources[cidx+1], 
                 (C.nSetSize - cidx - 1) * sizeof(resource_usage_type *) );
    }
    C.nSetSize --;

    if ( CTR.nCurrentRun == 0 ) {
        Check_CS();
        FreeCountMeasure(CTR.dCurrentTime, CNT); 
        FreeCountMeasure(CTR.dCurrentTime, CNT2); 
        AllocCountMeasure(CTR.dCurrentTime, &CNT); 
        AllocCountMeasure(CTR.dCurrentTime, &CNT2); 
    }

    BuildLambdaList();
    
    return;
}

/*******************************************************
        Callback interface
********************************************************/
int Add_Callback(int (*pCallBack)( double, int, void *), 
                      double dTime, int nParam, void *pParam,
                      unsigned int *CallBackID)
{
    CB.nSetSize ++;

    CB.callbacks = (fp_type *)realloc(
                    CB.callbacks, CB.nSetSize * 
                    sizeof(fp_type)  ); 

    if ( ! CB.callbacks ) {
        return ERR_CALLBACK_ALLOC;
    }

    /* register callback */
    CB.callbacks[CB.nSetSize - 1].f = pCallBack;
    *CallBackID = CB.nSetSize - 1;

    /* add to event list - NOTE!!! : period is multiple of time unit */
    dTime += CTR.dCurrentTime;
    AddCallBackEvent(CB.nSetSize-1, dTime, nParam, pParam);

#ifdef DEBUG
fprintf(stderr, "In Add_Callback_Reg() at %6.2f\n", dTime);
fflush(stderr);
#endif

    return ERR_SUCC;
}

void Remove_Callback(unsigned int CallBackID)
{
    /* assign a null function */
    if (CallBackID < 0 && CallBackID >= CB.nSetSize) {
        return;
    }

    CB.callbacks[CallBackID].f = null_cb;

    return;
}

/*******************************************************
        Rate management 
********************************************************/
void Set_Arrival_Rate(unsigned int nClassID, double dNewRate, double *dOldRate)
{
    int cidx;

    cidx = cid2idx(nClassID);
    if (cidx < ERR_SUCC) {
        return;
    }

	if (dOldRate != NULL) {
    	*dOldRate = C.dArrRate[cidx];
	}
    C.dArrRate[ cidx ] = dNewRate;
    BuildLambdaList();

    return;
}

void Set_Service_Rate(unsigned int nClassID, 
					  unsigned int resourceID, double dNewRate, 
                      double *dOldRate)
{
    int cidx, ridx;

    cidx = cid2idx(nClassID);
    if (cidx < ERR_SUCC) {
        return;
    }

    ridx = rid2cidx(nClassID, resourceID);
    if (ridx < ERR_SUCC) {
        return;
    }

	if (dOldRate != NULL) {
    	*dOldRate = C.sResources[cidx][ridx].dSerRate;
	}
    C.sResources[cidx][ridx].dSerRate = dNewRate;

    return;
}

/*******************************************************
        Dump management 
********************************************************/
void Register_Dump_Function( int (*pCallBack)(double, int, double**, double**,
double, void *), void *pDumpParam )
{
    DumpFunction = pCallBack;
    DumpParam = pDumpParam;
}

/*******************************************************
        Fetching metrics management 
********************************************************/
int Fetch_Avg_Basic_Metrics(int nMeasureID, unsigned int nClassID, 
                        unsigned int nResourceID, double *pValue) 
{
    int idx, cidx, ridx, i, k;
    double dN, dTime;

    dTime = CTR.dCurrentTime;
    assert(dTime < CTR.dSimulationTime);
    idx = rint(dTime / CTR.dCollectStdMeasure); 

    cidx = cid2idx_at(dTime, nClassID);
    if (cidx < ERR_SUCC) {
        return ERR_INVALID_CLASSID;
    }

    switch(nMeasureID) {
        case METRIC_ACCEPTED : 
            ridx = rid2cidx_at(dTime, nClassID, nResourceID);
            if (ridx < ERR_SUCC) {
                return ERR_INVALID_RESOURCEID;
            }
            *pValue = CNT[idx]->dAccepted_c_r[cidx][ridx];
            break;
        case METRIC_ACCEPTED_CLASS :
            *pValue = CNT[idx]->dAccepted_c[cidx];
            break;
        case METRIC_DEPART :
            ridx = rid2cidx_at(dTime, nClassID, nResourceID);
            if (ridx < ERR_SUCC) {
                return ERR_INVALID_RESOURCEID;
            }
            *pValue = CNT[idx]->dDepart_c_r[cidx][ridx];
            break;
        case METRIC_DEPART_CLASS :
            *pValue = CNT[idx]->dDepart_c[cidx];
            break;
        case METRIC_BLOCKED :
            ridx = rid2cidx_at(dTime, nClassID, nResourceID);
            if (ridx < ERR_SUCC) {
                return ERR_INVALID_RESOURCEID;
            }
            *pValue = CNT[idx]->dBlocked_c_r[cidx][ridx];
            break;
        case METRIC_BLOCKED_CLASS :
            *pValue = CNT[idx]->dBlocked_c[cidx];
            break;
        case METRIC_NUMBER :
            ridx = rid2cidx_at(dTime, nClassID, nResourceID);
            if (ridx < ERR_SUCC) {
                return ERR_INVALID_RESOURCEID;
            }
            /* *pValue = CNT[idx]->dNumber_c_r[cidx][ridx]; */
            *pValue = 0; 
            break;

        /* derived metrics */
        case METRIC_BLOCKING_PROB_CLASS :
            *pValue = CNT[idx]->dBlocked_c[cidx] /
                     ( CNT[idx]->dAccepted_c[cidx] + 
                       CNT[idx]->dBlocked_c[cidx] );
            break;

        case METRIC_BLOCKING_PROB :
            ridx = rid2cidx_at(dTime, nClassID, nResourceID);
            if (ridx < ERR_SUCC) {
                return ERR_INVALID_RESOURCEID;
            }
            *pValue = CNT[idx]->dBlocked_c_r[cidx][ridx] /
                     ( CNT[idx]->dPassed_c_r[cidx][ridx] + 
                       CNT[idx]->dBlocked_c_r[cidx][ridx] );
            break;

        case METRIC_NUMBER_CLASS :
            k = Get_CS(dTime);
            *pValue = 0.0;
            for (ridx=0;ridx < CS[k].nNumResources[cidx];ridx++) {
                /* *pValue += CNT[idx]->dNumber_c_r[cidx][ridx]; */
                *pValue += 0; 
            }
            break;

        case METRIC_UTILIZATION :
            dN = 0.0;
            for (i=0;i < idx;i++) {
                k = Get_CS(idx * CTR.dCollectStdMeasure);
                ridx = rid2cidx_at(idx * CTR.dCollectStdMeasure, 
                                   nClassID, nResourceID);
                if (ridx == ERR_SUCC) {
                    dN +=  CNT[idx]->dAccepted_c_r[cidx][ridx]
                           - CNT[idx]->dDepart_c_r[cidx][ridx];
                }
            }
            *pValue = ( dN * CS[k].sResources[cidx][ridx].dRequire ) /
                      CS[k].dCapacity[ridx];
            assert(*pValue <= 1.0);
            break;

        default :
            return ERR_INVALID_METRICID;
            break;
    }

    return ERR_SUCC;
}

int Fetch_Crt_Basic_Metrics(int nMeasureID, unsigned int nClassID, 
                        unsigned int nResourceID, double *pValue) 
{
    int idx, cidx, ridx, i, k;
    double dN, dTime;

    dTime = CTR.dCurrentTime;
    assert(dTime < CTR.dSimulationTime);
    idx = rint(dTime / CTR.dCollectStdMeasure); 

    cidx = cid2idx_at(dTime, nClassID);
    if (cidx < ERR_SUCC) {
        return ERR_INVALID_CLASSID;
    }

    switch(nMeasureID) {
        case METRIC_ACCEPTED : 
            ridx = rid2cidx_at(dTime, nClassID, nResourceID);
            if (ridx < ERR_SUCC) {
                return ERR_INVALID_RESOURCEID;
            }
            *pValue = CNT2[idx]->dAccepted_c_r[cidx][ridx];
            break;
        case METRIC_ACCEPTED_CLASS :
            *pValue = CNT2[idx]->dAccepted_c[cidx];
            break;
        case METRIC_DEPART :
            ridx = rid2cidx_at(dTime, nClassID, nResourceID);
            if (ridx < ERR_SUCC) {
                return ERR_INVALID_RESOURCEID;
            }
            *pValue = CNT2[idx]->dDepart_c_r[cidx][ridx];
            break;
        case METRIC_DEPART_CLASS :
            *pValue = CNT2[idx]->dDepart_c[cidx];
            break;
        case METRIC_BLOCKED :
            ridx = rid2cidx_at(dTime, nClassID, nResourceID);
            if (ridx < ERR_SUCC) {
                return ERR_INVALID_RESOURCEID;
            }
            *pValue = CNT2[idx]->dBlocked_c_r[cidx][ridx];
            break;
        case METRIC_BLOCKED_CLASS :
            *pValue = CNT2[idx]->dBlocked_c[cidx];
            break;
        case METRIC_NUMBER :
            ridx = rid2cidx_at(dTime, nClassID, nResourceID);
            if (ridx < ERR_SUCC) {
                return ERR_INVALID_RESOURCEID;
            }
            /* *pValue = CNT2[idx]->dNumber_c_r[cidx][ridx]; */
            *pValue = 0; 
            break;

        /* derived metrics */
        case METRIC_BLOCKING_PROB_CLASS :
            *pValue = CNT2[idx]->dBlocked_c[cidx] /
                     ( CNT2[idx]->dAccepted_c[cidx] + 
                       CNT2[idx]->dBlocked_c[cidx] );
            break;

        case METRIC_BLOCKING_PROB :
            ridx = rid2cidx_at(dTime, nClassID, nResourceID);
            if (ridx < ERR_SUCC) {
                return ERR_INVALID_RESOURCEID;
            }
            *pValue = CNT2[idx]->dBlocked_c_r[cidx][ridx] /
                     ( CNT2[idx]->dPassed_c_r[cidx][ridx] + 
                       CNT2[idx]->dBlocked_c_r[cidx][ridx] );
            break;

        case METRIC_NUMBER_CLASS :
            k = Get_CS(dTime);
            *pValue = 0.0;
            for (ridx=0;ridx < CS[k].nNumResources[cidx];ridx++) {
                /* *pValue += CNT2[idx]->dNumber_c_r[cidx][ridx]; */
                *pValue += 0; 
            }
            break;

        case METRIC_UTILIZATION :
            k = Get_CS(dTime);
            ridx = rid2cidx_at(dTime, nClassID, nResourceID);
            if (ridx < ERR_SUCC) {
                return ERR_INVALID_RESOURCEID;
            }
            dN = 0.0;
            for (i=0;i < idx;i++) {
                dN +=  CNT2[idx]->dAccepted_c_r[cidx][ridx]
                     - CNT2[idx]->dDepart_c_r[cidx][ridx];
            }
            *pValue = ( dN * CS[k].sResources[cidx][ridx].dRequire ) /
                      CS[k].dCapacity[ridx];
            /*assert(*pValue <= 1.0);*/
#ifdef DEBUG
			if (*pValue > 1.0) {
				fprintf(stderr, "pValue = %f\n", *pValue);
			}
#endif
            break;

        default :
            return ERR_INVALID_METRICID;
            break;
    }

    return ERR_SUCC;
}

/*******************************************************
        Print out error messsages 
********************************************************/
void Print_Mcmr_Error(int nErrcode)
{
    fprintf(stderr, "Error code = %d : %s\n", nErrcode, errmsg[-1 * nErrcode]);
}
