/***************************************************
  sys_mcmr.h : contains definitions for the time-dependent
  			   mcmr simulator
****************************************************/
#ifndef __SYS_MCMR_H
#define __SYS_MCMR_H

/* DEFINE CONSTANTS */
#define FALSE 0
#define TRUE 1

/* MACROS */
#define abs(x) ( (x)>0 ? (x) : (-1 * (x)) )
#define eventtype(x) ((x)->nEventType)
#define eventclass(x) ((x)->nClassIndex)
#define eventtimestamp(x) ((x)->dTimeStamp)

/* TYPE DEFINITIONS */
typedef struct resource_type_tag {
#define NOUSE 0
#define INUSE 1
	int nSetSize;			/* cardinality : number of resources */
	int *id;				/* resource ID array                 */
	int *inuse;				/* resource inuse array              */
	double *dCapacity;		/* double array : size = size, holds
                               maximum capacities of each resource */
	double *dAvailable;		/* double array : size = size, holds
                               available capacities of each resource */
    int **nClassId;         /* store classId's associated with each resource */
    int *nNumClasses;       /* store how many classes are associated with each resource */
} resource_type;

typedef struct resource_usage_type_tag {
	int nIndex;				/* index into the resources set */
	double dRequire;		/* amount of resource this class wants,
							   corresponds to the array resources[]
							   defined above */
	double dSerRate;		/* current service rates of the resources[]
							   being applied to this class customers  */  
} resource_usage_type; 

typedef struct class_type_tag {
	int nSetSize;			/* cardinality : number of classes */
	int *id;				/* class ID array                  */
	int *inuse;				/* class inuse array              */
	double *dArrRate;		/* current arrival rates of the classes */
	int *nNumResources;     /* number of resources each class requires */
	resource_usage_type **sResources; /* resources used by each class */ 
} class_type;

typedef struct checkpoint_struc_type_tag {
	int timestamp;
	int nCSetSize;
	int nRSetSize;
	int *cid;
	int *rid;
	int *nNumResources;
	double *dCapacity;
	resource_usage_type **sResources; /* resources used by each class */ 
} checkpoint_struc_type; 

typedef struct std_count_type_tag {
#define COUNT_ACCEPTED 		 0
#define COUNT_ACCEPTED_CLASS 1
#define COUNT_DEPART  		 2
#define COUNT_DEPART_CLASS   3
#define COUNT_BLOCKED 		 4
#define COUNT_BLOCKED_CLASS  5
#define COUNT_PASSED 		 6
	double **dAccepted_c_r;			
	double *dAccepted_c;
	double **dDepart_c_r;
	double *dDepart_c;
	double **dBlocked_c_r;
	double *dBlocked_c;
	double **dPassed_c_r;			/* number passed the admission test */
} std_count_type;

typedef struct fp_tag {
    int (*f)(double, int, void *);
} fp_type;

typedef struct callback_type_tag {
	int nSetSize;
	fp_type *callbacks;
} callback_type;

typedef struct control_type_tag {
	double dSimulationTime;    /* total simulation time */
	double dCurrentTime;       /* current simulation time */ 
	int nCurrentEventRecord;   /* current event record */
	int nCurrentMeasureRecord; /* current measure record */
	double dCollectStdMeasure; /* interval to collect standard measures */
	int nSimulationRuns;	   /* number of simulation runs */
	int nCurrentRun;	   	   /* current run : from 0 */ 
} control_type;

typedef struct event_type_tag {
#define UNUSED       0
#define ARRIVAL      1
#define DEPARTURE    2
#define CALLBACK     3
#define CALLBACKINIT 4
	int nEventType;			 /* type of event */
	double dTimeStamp;
	int nClassIndex;	
	struct event_type_tag *pNext, *pPrev;
} event_type;

typedef struct event_list_type_tag {
/*
#define MAXEVENTS 500000
#define EVENT_BOUND (MAXEVENTS + 10000)  
#define MAXDEVIATE 100        
	event_type list[MAXEVENTS];
*/
	event_type header;
	event_type *hp;
} event_list_type;

typedef struct cb_event_type_tag {
	int nId;
	int nParam;
	void *pParam;
} cb_event_type;

/* EXTERNAL DECLARATIONS */
extern resource_type R, R2;
extern class_type C, C2;
extern control_type CTR;
extern callback_type CB;
extern std_count_type **CNT;
extern std_count_type **CNT2;
event_list_type E;
checkpoint_struc_type *CS;

extern char *errmsg[];

extern double *Lambdas;
extern double dLambda;

extern int ( *ArrRateFunction )( double, int, double* );
extern int ( *SerRateFunction )( double, int, int, double* );
extern int ( *DumpFunction )( double, int, double**, double**, double, void * );

extern void *DumpParam;


extern void AllocCountMeasure(double, std_count_type ***);
extern void FreeCountMeasure(double, std_count_type **);
extern void ClearCountMeasure();
extern void Set_ClassId();
extern int cid2idx(int);
extern int rid2idx(int);
extern int rid2cidx(int, int);
extern int idx2cid(int);
extern int idx2rid(int);
extern int cid2idx_at(double, int);
extern int rid2idx_at(double, int);
extern int rid2cidx_at(double, int, int);

extern double expntl(double);
extern double drand48();
extern double rint(double);

extern void InitEvent();
extern void AddEvent(int, int);
extern void DeleteEvent(event_type *);
extern event_type *FetchNextEvent();
extern event_type *GetNewIndex();
extern void AdvanceEventlistHdr(event_type *);
extern void ResetEventlistHdr();

extern void BuildLambdaList();

extern void FreeC(class_type *);
extern void FreeR(resource_type *);

extern int null_cb(double, int, void *);

#endif
