#define TRUE 1
#define FALSE 0

#define MAX_NUM_ITER 500

#define SEL_HOP       1
#define SEL_UTIL      2
#define SEL_UTIL_HOP  3
#define MAX_NODES     500

#define MAX_NODE_GRP  50
#define MAX_LINK_GRP  50

#define LINK_AGR_TYPE0 0
#define LINK_AGR_TYPE1 1

#define LEVEL0		   0
#define LEVEL1		   1

#define min(x,y) (((x) > (y)) ? y : x)
#define max(x,y) (((x) > (y)) ? x : y)

/* 
 * the class characteristics :
 */

typedef struct tagTRAFFIC_CLASS {
  unsigned int nID;
  
  double dL;                    /* arrival rate */
  double dLTime;                /* variation time for lambda */

  double dWeight;
  double dOldWeight;

  double dM;                    /* serving time */

  unsigned int nReserv;         /* required Bandwidth */
  unsigned int nHops;

  unsigned int nLinks;
  unsigned int *nBelong;        /* path from the src to dest */
  double *dBlocking;            /* blocking prob. for each link */

  double *dZ;                   /* current z value for each link */ 
  double *dZOld;                /* old z value for each link */

  double dN;                    /* current number of traffics */
  double dNOld;                 /* old number of traffics */

  double dTotalBlocking;

  unsigned int *nPath;			/* level-0 path             */
  unsigned int serv_idx;		/* Back link to the service */	
  unsigned int intclass_idx;	/* Back link to the interclass */	

} TRAFFIC_CLASS;

/* 
 * the intermediate class that holds level-1 paths 
 */
typedef struct tagINTER_CLASS {
	unsigned int viewidx;		/* the view index the paths are built from */
	unsigned int numpNodes;     /* number of elements on the path */ 
	struct pNode_tag {          /* holds path nodes */
		unsigned int id;
		unsigned int lv;
	} *pNodes;
	unsigned int serv_idx;		/* index of service associated with */
	unsigned int *class_idx;	/* indice of classes associated with */
	unsigned int nclasses;		/* numberof classes associated with */
} INTER_CLASS;

typedef struct tagOUTER_CLASS {

  double dL; /* arrival rate */
  double dLTime; /* variation time for lambda */
  double dM; /* serving time */

  double dTotalWeight;

  /* From where starts and where ends the connection */
  unsigned int nStartNode;
  unsigned int nStopNode;

  /* The bandwidth stuff */
  unsigned int nReserv;

  /* the number of inner classes */
  unsigned int nInnerClass;
  /* The inner classes */
  TRAFFIC_CLASS** pInnerClass;

  /* the number of intermediate classes : level-1 paths */
  unsigned int nInterClass;
  /* The intermediate classes */
  INTER_CLASS** pInterClass;

  /* The minimum number of hops */
  unsigned int nMinHops;
  unsigned int nLvOneMinHops;
} OUTER_CLASS;

typedef struct tagLEVEL_ZERO_LINK {
	unsigned int s0; 				/* level-0 */
	unsigned int e0;				/* level-0 */
	unsigned int s;					/* level-1 */
	unsigned int e;					/* level-1 */
} LEVEL_ZERO_LINK;

typedef struct tagLEVEL_ONE_NODE {
	unsigned int num;
	unsigned int nz[MAX_NODE_GRP];
} LEVEL_ONE_NODE;

typedef struct tagLEVEL_ONE_LINK {
	unsigned int num;
	unsigned int type;
	/* id is either (x,y) or (x.n, y) -> TYPE0, TYPE1 */ 
	struct id_tag {
		unsigned int s0;			/* level-1 */
		unsigned int s1;			/* level-0 */
		unsigned int e;				/* level-1 */
	} id;
	unsigned int nz[MAX_LINK_GRP];
} LEVEL_ONE_LINK;

typedef struct tagVIEW { /* number of views = nLvOneNodes */
	unsigned int nNodes;
	unsigned int nLvZeroNodes;
	struct tag_vn {
		unsigned int id;
		unsigned int lv;
	} *pNd;
	struct tag_vl {
		unsigned int id;
		unsigned int lv;
	} *pAj;
} VIEW;

