/***************************************************
    model-multi.c : main part of the simulator
****************************************************/
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "mcmr.h"

int dcb(double, int, double, double);
int dumper(double, int, double**, double**, double, void *);

double stime;
double capacity[] = {150.0, 200.0, 250.0};
double arrrate[] = {0.125, 0.5, 0.2, 0.1, 0.125,
                    0.5, 1.0, 0.0625, 1.0, 0.25,
                    0.125, 0.5, 0.2, 0.1, 0.125,
                    0.5, 1.0, 0.0625, 1.0, 0.25 };

main(int argc, char **argv)
{
    int i, runs;

    if (argc != 3) {
        fprintf(stderr, "usage: %s simulation_runs simulation_time\n",
                argv[0]);
        exit(1);
    }
    sscanf(argv[1], "%d", &runs);
    sscanf(argv[2], "%lf", &stime);

    Init_Q_Module();

    Set_Config_d(CONFIG_SIMULATION_TIME, stime);
    Set_Config_d(CONFIG_DUMP_PERIOD, 0.1);
    Set_Config_i(CONFIG_NUMBER_OF_SIMULATIONS, runs);
    Set_Config_d(CONFIG_EVALUATION_PERIOD, 0.1);
    Set_Config_d(CONFIG_EPSILON_STOP, 0.0001);

    for (i=0;i < 3;i++) {
        Add_Resource(i, capacity[i]); 
    }
    for (i=0;i < 20;i++) {
        Add_Class(i, arrrate[i]); 
    }

    for (i=0;i < 3;i++) {
        Add_Class_At_Resource(0, i, 30.0, 0.2); 
    }

    for (i=0;i < 3;i++) {
        Add_Class_At_Resource(1, i, 15.0, 1.0); 
    }

    for (i=0;i < 3;i++) {
        Add_Class_At_Resource(2, i, 50.0, 0.5); 
    }

    for (i=0;i < 3;i++) {
        Add_Class_At_Resource(3, i, 10.0, 0.5); 
    }

    for (i=0;i < 3;i++) {
        Add_Class_At_Resource(4, i, 40.0, 1.0); 
    }

    for (i=0;i < 3;i++) {
        Add_Class_At_Resource(5, i, 25.0, 2.0); 
    }

    for (i=0;i < 3;i++) {
        Add_Class_At_Resource(6, i, 30.0, 2.0); 
    }

    for (i=0;i < 3;i++) {
        Add_Class_At_Resource(7, i, 10.0, 0.1); 
    }

    for (i=0;i < 3;i++) {
        Add_Class_At_Resource(8, i, 5.0, 5.0); 
    }

    for (i=0;i < 3;i++) {
        Add_Class_At_Resource(9, i, 50.0, 0.5); 
    }

    Add_Class_At_Resource(10, 0, 30.0, 0.2);
    Add_Class_At_Resource(11, 0, 15.0, 1.0);
    Add_Class_At_Resource(12, 0, 50.0, 0.5);
    Add_Class_At_Resource(13, 1, 10.0, 0.5);
    Add_Class_At_Resource(14, 1, 40.0, 1.0);
    Add_Class_At_Resource(15, 1, 25.0, 2.0);
    Add_Class_At_Resource(16, 2, 30.0, 2.0);
    Add_Class_At_Resource(17, 2, 10.0, 0.1);
    Add_Class_At_Resource(18, 2,  5.0, 5.0);
    Add_Class_At_Resource(19, 2, 50.0, 0.5);

    Register_Dump_Function( dumper, NULL );

    Evaluate();
    Done_Q_Module();
}

int dcb(double time, int classid, double d1, double d2)
{
    int i;

    double dAccepted, dBlocked, dDeparted, dBp, value;
    static double dN = 0.0;

    if (time < stime - 0.5) {
        dAccepted =  dBlocked =  dDeparted = 0.0;
        for (i=0;i < 20;i++) {
            Fetch_Avg_Basic_Metrics(METRIC_ACCEPTED_CLASS, i, 0, &value);
            dAccepted += value;
            Fetch_Avg_Basic_Metrics(METRIC_DEPART_CLASS, i, 0, &value);
            dDeparted += value;
            Fetch_Avg_Basic_Metrics(METRIC_BLOCKED_CLASS, i, 0, &value);
            dBlocked += value;
        }
        dN += (dAccepted - dDeparted);
        dBp = (dAccepted + dBlocked) != 0.0 ?
                dBlocked / (dAccepted + dBlocked): 0.0;
        printf("%6.2f %10.5f %10.5f %10.5f %10.5f %10.5f\n",
            time,
            dAccepted / 0.1, dDeparted, dBlocked, dN,
            dBp);
    }
}

int dumper( double crtTime, int classID, double* dN[],
            double* dBlocking[], double  dAccess, void * pDumpParam)
{
  static double dTime;
  static double dTotalN;
  static double dThroughput;

  /* Another dump cycle */
  if( dTime != crtTime ) {
    if( 0.0 != dTime )
      printf( "%lf:%lf:%lf\n", dTime, dTotalN, dThroughput );
    dTotalN = 0.0;
    dThroughput = 0.0;
    dTime = crtTime;
  }

  dTotalN += *dN[0];
  dThroughput += arrrate[ classID ] * dAccess;

  return 0;
}
