/***************************************************
    main.c : main part of the simulator
****************************************************/
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "mcmr.h"

int cb(double, int, void*);
int dcb(double, int, double, double);
int dumper(double, int, double**, double**, double, void *);

double stime;
int cid[] = {0,1,2,3};
int rid[] = {0,1,2};
double capacity[] = {50.0, 100.0, 150.0};
double arrrate[] = {0.0, 0.125, 0.1, 0.25};
double require[] = {30.0, 30.0, 30.0};
double serrate[] = {0.2, 0.2, 0.2};

main(int argc, char **argv)
{
    int callID, runs, i;

    if (argc != 3) {
        fprintf(stderr, "usage: %s simulation_runs simulation_time\n",
                argv[0]);
        exit(1);
    } 
    sscanf(argv[1], "%d", &runs);
    sscanf(argv[2], "%lf", &stime);

    Init_Q_Module();

    Set_Config_d(CONFIG_SIMULATION_TIME, stime);
    Set_Config_d(CONFIG_DUMP_PERIOD, 0.1);
    Set_Config_i(CONFIG_NUMBER_OF_SIMULATIONS, runs);
    Set_Config_d(CONFIG_EVALUATION_PERIOD, 0.1);
    Set_Config_d(CONFIG_EPSILON_STOP, 0.0001);

    for (i=0;i < 3;i++) {
        Add_Resource(rid[i], capacity[i]);
    }
    for (i=0;i < 4;i++) {
        Add_Class(cid[i], arrrate[i]);
    }

    for (i=0;i < 3;i++) {
        Add_Class_At_Resource(cid[0], rid[i], require[i], serrate[i]);
    }
    Add_Class_At_Resource(cid[1], rid[0], 30.0, 0.2);
    Add_Class_At_Resource(cid[2], rid[1], 10.0, 0.5);
    Add_Class_At_Resource(cid[3], rid[2], 50.0, 0.5);

    Register_Dump_Function( dumper, NULL );

    Add_Callback( cb, 20.00, 1, "abcde", &callID );
    Add_Callback( cb, 40.00, 1, "abcde", &callID );

    Evaluate();

    Done_Q_Module();
}

int cb( double time, int nParam, void *pParam)
{
    double oldRate;

    Set_Arrival_Rate(cid[0], 0.0, &oldRate);
    if (oldRate == 0.0 && time != 0.0) {
        Set_Arrival_Rate(cid[0], 0.125, &oldRate);
    } else {
        Set_Arrival_Rate(cid[0], 0.0, &oldRate);
    }
#ifdef DEBUG
fprintf(stderr, "I was called at %f, %d, %s\n", time, nParam, pParam); 
fflush(stderr);
#endif
}

int dcb(double time, int classid, double d1, double d2) 
{
    int i;

    double dAccepted, dBlocked, dDeparted, dBp, value;
    static double dN = 0.0;

    if (time < stime - 0.5) {
        dAccepted =  dBlocked =  dDeparted = 0.0;
        for (i=4;i <= 7;i++) {
            Fetch_Avg_Basic_Metrics(METRIC_ACCEPTED_CLASS, i, 0, &value);
            dAccepted += value;
            Fetch_Avg_Basic_Metrics(METRIC_DEPART_CLASS, i, 0, &value);
            dDeparted += value; 
            Fetch_Avg_Basic_Metrics(METRIC_BLOCKED_CLASS, i, 0, &value);
            dBlocked += value; 
        }
        dN += (dAccepted - dDeparted);
        dBp = (dAccepted + dBlocked) != 0.0 ?
                dBlocked / (dAccepted + dBlocked): 0.0;
        printf("%6.2f %10.5f %10.5f %10.5f %10.5f %10.5f\n",
            time,
            dAccepted / 0.1, dDeparted, dBlocked, dN,
            dBp);
    }
}

int dumper( double crtTime, int classID, double* dN[],
            double* dBlocking[], double  dAccess, void * pDumpParam)
{
  static double dTime;
  static double dTotalN;
  static double dThroughput;

  /* Another dump cycle */
  if( dTime != crtTime ) {
    if( 0.0 != dTime )
      printf( "%lf:%lf:%lf\n", dTime, dTotalN, dThroughput );
    dTotalN = 0.0;
    dThroughput = 0.0;
    dTime = crtTime;
  }

  dTotalN += *dN[0];
  dThroughput += arrrate[ classID ] * dAccess;

  return 0;
}
